@@ -8,7 +8,6 @@
 #include "book.h"
 #include "bookview.h"
 #include "library.h"
-#include "selectionsuppressor.h"
 #include "settings.h"
 #include "trace.h"
 
@@ -19,20 +18,29 @@
 #endif
 
 BookView::BookView(QWidget *parent):
-    QWebView(parent), contentIndex(-1), mBook(0), restore(true),
-    positionAfterLoad(0), loaded(false)
+    QWebView(parent), contentIndex(-1), mBook(0), restorePositionAfterLoad(false),
+    positionAfterLoad(0), loaded(false), contentsHeight(0), decorated(false)
 {
     Trace t("BookView::BookView");
     settings()->setAttribute(QWebSettings::AutoLoadImages, true);
     settings()->setAttribute(QWebSettings::JavascriptEnabled, true);
+    settings()->setAttribute(QWebSettings::JavaEnabled, false);
     settings()->setAttribute(QWebSettings::PluginsEnabled, false);
+    settings()->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
+    settings()->setAttribute(QWebSettings::JavascriptCanOpenWindows, false);
+    settings()->setAttribute(QWebSettings::JavascriptCanAccessClipboard, false);
+    settings()->setAttribute(QWebSettings::OfflineStorageDatabaseEnabled, false);
+    settings()->setAttribute(QWebSettings::OfflineWebApplicationCacheEnabled, false);
+    settings()->setAttribute(QWebSettings::LocalStorageEnabled, false);
     settings()->setAttribute(QWebSettings::ZoomTextOnly, true);
     settings()->setAttribute(QWebSettings::LocalContentCanAccessRemoteUrls,
                              false);
+    settings()->setDefaultTextEncoding("utf-8");
     page()->setContentEditable(false);
 
 #if defined(Q_WS_MAEMO_5)
-    (void)new SelectionSuppressor(this);
+    // Suppress unwanted text selections on Maemo
+    installEventFilter(this);
 #endif
     QWebFrame *frame = page()->mainFrame();
 #if defined(Q_WS_MAEMO_5)
@@ -43,6 +51,10 @@ BookView::BookView(QWidget *parent):
     bookmarkImage = QImage(":/icons/bookmark.png");
 
     connect(this, SIGNAL(loadFinished(bool)), this, SLOT(onLoadFinished(bool)));
+    connect(frame, SIGNAL(javaScriptWindowObjectCleared()),
+            this, SLOT(addJavaScriptObjects()));
+    connect(frame, SIGNAL(contentsSizeChanged(const QSize &)),
+            this, SLOT(onContentsSizeChanged(const QSize &)));
     connect(Settings::instance(), SIGNAL(valueChanged(const QString &)),
             this, SLOT(onSettingsChanged(const QString &)));
     Settings *s = Settings::instance();
@@ -84,6 +96,7 @@ void BookView::loadContent(int index)
     }
     else {
         loaded = false;
+        decorated = false;
         emit chapterLoadStart(index);
         load(QUrl(contentFile));
     }
@@ -93,13 +106,17 @@ void BookView::loadContent(int index)
 void BookView::setBook(Book *book)
 {
     Trace t("BookView::setBook");
+
+    // Save position in current book
     setLastBookmark();
+
+    // Open new book, restore last position
     if (book != mBook) {
         mBook = book;
         if (book) {
             contentIndex = -1;
             book->open();
-            goToBookmark(book->lastBookmark());
+            restoreLastBookmark();
         }
         else {
             contentIndex = 0;
@@ -127,45 +144,50 @@ void BookView::goNext()
 
 void BookView::setLastBookmark()
 {
-    Trace t("BookView::saveLastBookmark");
+    Trace t("BookView::setLastBookmark");
     if (mBook) {
-        int height = page()->mainFrame()->contentsSize().height();
+        int height = contentsHeight;
         int pos = page()->mainFrame()->scrollPosition().y();
+        t.trace(QString("At %1 (%2%, height %3)").
+                arg(pos).arg((qreal)pos / (qreal)height * 100).arg(height));
         mBook->setLastBookmark(contentIndex, (qreal)pos / (qreal)height);
     }
 }
 
+void BookView::restoreLastBookmark()
+{
+    Trace t("BookView::restoreLastBookmark");
+    if (mBook) {
+        goToBookmark(mBook->lastBookmark());
+    }
+}
+
 void BookView::goToBookmark(const Book::Bookmark &bookmark)
 {
     Trace t("BookView::goToBookmark");
     if (mBook) {
-        restore = true;
-        positionAfterLoad = bookmark.pos;
         if (bookmark.chapter != contentIndex) {
+            t.trace(QString("Loading new chapter %1").arg(bookmark.chapter));
+            restorePositionAfterLoad = true;
+            positionAfterLoad = bookmark.pos;
             loadContent(bookmark.chapter);
         } else {
-            onLoadFinished(true);
+            goToPosition(bookmark.pos);
         }
     }
 }
 
 void BookView::onLoadFinished(bool ok)
 {
-    Trace t(QString("BookView::onLoadFinished: %1").arg(ok));
-    loaded = true;
-    if (ok) {
-        addNavigationBar();
+    Trace t("BookView::onLoadFinished");
+    if (!ok) {
+        t.trace("Not OK");
+        return;
     }
+    loaded = true;
+    addNavigationBar();
     onSettingsChanged("scheme");
     emit chapterLoadEnd(contentIndex);
-    if (restore) {
-        restore = false;
-        if (ok && mBook) {
-            int height = page()->mainFrame()->contentsSize().height();
-            int scrollPos = (qreal)height * positionAfterLoad;
-            page()->mainFrame()->setScrollPosition(QPoint(0, scrollPos));
-        }
-    }
 }
 
 void BookView::onSettingsChanged(const QString &key)
@@ -208,7 +230,7 @@ void BookView::paintEvent(QPaintEvent *e)
         if (b.chapter != contentIndex) {
             continue;
         }
-        int height = page()->mainFrame()->contentsSize().height();
+        int height = contentsHeight;
         int bookmarkPos = (qreal)height * (qreal)b.pos;
         painter.drawPixmap(2, bookmarkPos - scrollPos.y(), bookmarkPixmap);
     }
@@ -234,7 +256,7 @@ void BookView::addBookmark()
     int height = page()->mainFrame()->contentsSize().height();
     t.trace(QString().setNum((qreal)y / (qreal)height));
     mBook->addBookmark(contentIndex, (qreal)y / (qreal)height);
-    repaint();
+    update();
 }
 
 void BookView::addNavigationBar()
@@ -266,7 +288,6 @@ void BookView::addNavigationBar()
     }
 
     QWebFrame *frame = page()->currentFrame();
-    frame->addToJavaScriptWindowObject("bv", this);
     QString headerScript = "document.body.innerHTML = '" +
         naviPrev + naviNext + "<br />" + "' + document.body.innerHTML;";
     QString trailerScript = "document.body.innerHTML += '<br /><br />" +
@@ -274,6 +295,7 @@ void BookView::addNavigationBar()
 
     frame->evaluateJavaScript(headerScript);
     frame->evaluateJavaScript(trailerScript);
+    decorated = true;
 }
 
 QString BookView::tmpPath()
@@ -297,3 +319,82 @@ void BookView::removeIcons()
     QFile(ICON_PREFIX + QString("/previous.png")).remove();
     QDir().rmpath(tmpPath());
 }
+
+bool BookView::eventFilter(QObject *o, QEvent *e)
+{
+#if 0
+    if (e->type() != QEvent::Paint && e->type() != QEvent::MouseMove) {
+        if (e->type() == QEvent::Resize) {
+            Trace::debug(QString("BookView::eventFilter QEvent::Resize to %1").
+                         arg(page()->mainFrame()->contentsSize().height()));
+        } else {
+            Trace::debug(QString("BookView::eventFilter %1").
+                         arg(Trace::event(e->type())));
+        }
+    }
+#endif
+
+    switch (e->type()) {
+    case QEvent::MouseButtonPress:
+        emit suppressedMouseButtonPress();
+        mousePressed = true;
+        break;
+    case QEvent::MouseButtonRelease:
+        mousePressed = false;
+        break;
+    case QEvent::MouseMove:
+        if (mousePressed) {
+            return true;
+        }
+        break;
+    case QEvent::MouseButtonDblClick:
+        return true;
+    default:
+        break;
+    }
+
+    return QObject::eventFilter(o, e);
+}
+
+void BookView::addJavaScriptObjects()
+{
+    page()->mainFrame()->addToJavaScriptWindowObject("bv", this);
+}
+
+void BookView::onContentsSizeChanged(const QSize &size)
+{
+    contentsHeight = size.height();
+    if (decorated) {
+        if (restorePositionAfterLoad) {
+            Trace::debug("BookView::onContentSizeChanged: Time to restore");
+            restorePositionAfterLoad = false;
+            goToPosition(positionAfterLoad);
+        }
+    }
+}
+
+void BookView::leaveEvent(QEvent *e)
+{
+    Trace t("BookView::leaveEvent");
+    // Save current position, to be restored later
+    setLastBookmark();
+    QWebView::leaveEvent(e);
+}
+
+void BookView::enterEvent(QEvent *e)
+{
+    Trace t("BookView::enterEvent");
+    // Restore position saved at Leave event. This seems to be required,
+    // after temporarily switching from portrait to landscape and back
+    restoreLastBookmark();
+    QWebView::enterEvent(e);
+}
+
+void BookView::goToPosition(qreal position)
+{
+    int scrollPos = (qreal)contentsHeight * position;
+    page()->mainFrame()->setScrollPosition(QPoint(0, scrollPos));
+    // FIXME: update();
+    Trace::debug(QString("BookView::goToPosition: To %1 (%2%, height %3)").
+            arg(scrollPos).arg(position * 100).arg(contentsHeight));
+}
